/* Wrap std::istreams in QIODevices.
 * Copyright (C) 2006  Benoit Sigoure.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */

#ifndef QSTDIODEVICE_HH_
# define QSTDIODEVICE_HH_

# include <iosfwd>
# include <QIODevice>

/// Adapter: adapts an @c std::istream as a @c QIODevice.
class QStdIStream: public QIODevice
{
public:
  QStdIStream (std::istream& is);

  /*******************************************
  * Virtual members inherited from QIODevice *
  *******************************************/

  /// Dtor.
  virtual ~QStdIStream ();

  /** Tells whether or not the current read and write position is at the end
   *  of the device.
   *
   *  For some devices, @c atEnd can return @c true even though there is more
   *  data to read. This special case only applies to devices that generate
   *  data in direct response to you calling @c read (e.g., /dev or /proc files
   *  on Unix and Mac OS X, or console input / @c stdin on all platforms).
   *
   *  @return @c true if the current read and write position is at the end of
   *  the device. (i.e. there is no more data available for reading on the
   *  device); otherwise returns @c false.
   *
   *  @see bytesAvailable, read, isSequential.
   */
  virtual bool atEnd () const;

protected:
  /** Reads up to @a maxSize bytes from the device into @a data.
   *  @return the number of bytes read or -1 if an error occurred.
   */
  virtual qint64 readData (char* data, qint64 maxSize);

  /** You can't write to a @c std::istream.
   *  @return -1.
   */
  virtual qint64 writeData (const char* data, qint64 maxSize);

private:
  std::istream& stream_;
};

# include "QStdIStream.hxx"

#endif /* !QSTDIODEVICE_HH_ */
